
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import scipy.ndimage as spImg
import math




def createRTable(model):

    h, w = model.shape

    # Mittelpunkt finden:
    cx = -1
    cy = -1
    k = 0
    for y in xrange(h):
        for x in xrange(w):
            if(model[y, x] > 0): # Vordergrundpixel
                k += 1
                cx += x
                cy += y
    cx /= k
    cy /= k
    
    
    # Gradienten berechnen:
    Gx = spImg.filters.sobel(model, 1)
    Gy = spImg.filters.sobel(model, 0)
    
    
    # Tabelle anlegen und fuellen:
    rTable = {}
    for y in xrange(h):
        for x in xrange(w):
            if(model[y, x] > 0): # Vordergrundpixel
                theta = int(math.degrees(math.atan2(Gy[y, x], Gx[y, x])))
                dx = cx - x
                dy = cy - y
                if(rTable.has_key(theta)):
                    rTable[theta].append((dx, dy))
                else:
                    rTable.setdefault(theta, [(dx, dy)])
    
    return rTable



def houghTransform(test, model):
    
    # Preprocessing - Binaerisieren:
    m = np.max(test) * 0.6
    test = np.array([[ (0 if v < m else 1) for v in row] for row in test], dtype="int")
    m = np.max(model) * 0.6
    model = np.array([[ (0 if v < m else 1) for v in row] for row in model], dtype="int")
    
    # RTable berechnen:
    rTable = createRTable(model)
    
    # Gradienten berechnen:
    Gx = spImg.filters.sobel(test, 1)
    Gy = spImg.filters.sobel(test, 0)
        
    h, w = test.shape
    result = np.zeros((h, w))
    
    for y in xrange(h):
        print y
        for x in xrange(w):
            if(test[y, x] > 0): # Vordergrundpixel
                theta = int(math.degrees(math.atan2(Gy[y, x], Gx[y, x])))
                if(rTable.has_key(theta)):
                    for d in rTable[theta]:
                        xinc = x + d[0]
                        yinc = y + d[1]
                        if(xinc > 0 and xinc < w and yinc > 0 and yinc < h):
                            result[yinc, xinc] += math.sqrt(Gx[y, x] ** 2 + Gy[y, x] ** 2)
    
    return result          
                
                

def nonMaximumSuppression(image):
    
    h, w = image.shape
    imageClean = np.zeros_like(image)
    radius = 4
    for y in xrange(radius, h - radius):
        for x in xrange(radius, w - radius):
            part = image[y - radius:y + radius, x - radius:x + radius]
            maxVal = np.max(part)
            if maxVal == image[y, x]:
                imageClean[y, x] = image[y, x]
    
    return imageClean     



def aufgabe2():
    
    model = plt.imread('generalhoughmodel.png')
    test = plt.imread('generalhoughtest.png')
    
        
    result = houghTransform(test, model)
    
    resultThreshold = result.copy()
    resultThreshold[resultThreshold < 0.75 * np.max(resultThreshold)] = 0
    resultClean = nonMaximumSuppression(resultThreshold)
    
    
    plt.figure(1)
    plt.subplot(221)
    plt.imshow(model, cmap=plt.cm.gray)
    plt.subplot(222)
    plt.imshow(test, cmap=plt.cm.gray)
    plt.subplot(223)
    plt.imshow(result, cmap=plt.cm.gray)
    plt.subplot(224)
    plt.imshow(resultClean, cmap=plt.cm.gray)
    plt.show()
    
    

aufgabe2()
