
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import scipy.ndimage as spImg
import math




def preprocess(image):
    
    # Tiefpassfilterung:
    result = spImg.gaussian_filter(image, 1.75)
    
    # Gradient:
    Gx = spImg.filters.sobel(result, 1)
    Gy = spImg.filters.sobel(result, 0)  

    return Gx, Gy



def houghTransform(Gx, Gy):
      
    # Hough-Raum diskretisiert "anlegen":
    h, w = Gx.shape
    dist = 2 * max(h, w)#math.ceil(math.sqrt(h ** 2 + w ** 2))
    maxAngle = 150
    houghDomain = np.zeros((maxAngle, dist))
      
    for y in xrange(h):
        print y
        for x in xrange(w):
            #if(image[y, x] == 0):
            angle = math.atan2(Gy[y, x], Gx[y, x]) if Gx[y, x] != 0 else 0         
            d = dist / 3.0 + int(x * math.cos(angle) + y * math.sin(angle))
            angleDeg = int(math.degrees(angle))
            
            if(angleDeg > 0 and angleDeg < maxAngle):
                houghDomain[angleDeg, d] += math.sqrt(Gx[y, x] ** 2 + Gy[y, x] ** 2)
                             
   
    return houghDomain



def findLines(houghDomain):
    
    h, w = houghDomain.shape    
    
    result = np.zeros((h))
    for a in xrange(h):
        result[a] = np.sum(houghDomain[a, :])
                
    angle = np.argmax(result)
       
    lines = []
    for a in xrange(h):
        for d in xrange(w):
            if(a == angle):
                lines.append((a, d, houghDomain[a, d]))
                
    lines = sorted(lines, key=lambda x: x[2], reverse=True)[0:min(len(lines), 60)]
       
    return result, lines



def underlineText(image, lines):
    
    h, w = image.shape
       
    for line in lines:
        
        angleRad = math.radians(line[0])
        dist = line[1] - 2 * max(h, w) / 3.0
        cosVal = math.cos(angleRad)
        sinVal = math.sin(angleRad)
        
        xVals = np.arange(0, w, 1)
        yVals = (dist - xVals * cosVal) / sinVal if sinVal != 0 else 0
        plt.plot(xVals, yVals, "r")
    
   

def aufgabe1a():
    
    input = plt.imread('paper.png')
    Gx, Gy = preprocess(input)
    result = houghTransform(Gx, Gy)
    
    anglePlot, lines = findLines(result)
    
    plt.figure(1)
    plt.subplot(221)
    plt.imshow(np.sqrt(Gx ** 2 + Gy ** 2), cmap=plt.cm.gray)
    plt.subplot(222)
    plt.imshow(result, cmap=plt.cm.gray)
    plt.subplot(223)
    plt.plot(np.arange(0, 150, 1), anglePlot, color='r')
    plt.subplot(224)
    plt.imshow(input, cmap=plt.cm.gray)
    underlineText(input, lines)
    plt.show()
    
    

aufgabe1a()
